@@ -1,14 +1,14 @@
 use std::ops::{Add, AddAssign};
 use std::time::Duration;
 
-use util::NiceDuration;
+use crate::util::NiceDuration;
 
 /// Summary statistics produced at the end of a search.
 ///
 /// When statistics are reported by a printer, they correspond to all searches
 /// executed with that printer.
 #[derive(Clone, Debug, Default, PartialEq, Eq)]
-#[cfg_attr(feature = "serde1", derive(Serialize))]
+#[cfg_attr(feature = "serde1", derive(serde::Serialize))]
 pub struct Stats {
     elapsed: NiceDuration,
     searches: u64,
